/*
 * Xgraph parameters
 */

#ifndef _PARAMS_H_
#define _PARAMS_H_

#include "tcl.h"
#include "tk.h"
#include "nxgraph.h"
#include "st.h"


#define BITDATA(s) ((char *)&((s)[0]))
#define TKXGBITMAPROOT "tkxgraph_bits_"

typedef enum param_types_defn {
ERR, INT, STR, PIXEL, FONT, STYLE, BOOL, DBL 
} param_types;

typedef struct params_int_defn {
    param_types type;		/* INT */
    int value;
} param_int;

typedef struct params_str_defn {
    param_types type;		/* STR */
    char *value;
} param_str;

typedef struct params_pix_defn {
    param_types type;		/* PIXEL */
    XColor *value;	/* tk get color return */
    char *tkcolorname;  /* name from user */
} param_pix;

typedef struct params_font_defn {
    param_types type;		/* FONT */
    Tk_Font value;
    char *tkfontname;
    int tkfontnamelen;
} param_font;

typedef struct params_style_defn {
    param_types type;		/* STYLE */
    int len;
    char *dash_list;
    char *tkbitname; /* we own this. tk has a uid and owns the bitdata */
} param_style;

typedef struct params_bool_defn {
    param_types type;		/* BOOL */
    int value;
} param_bool;

typedef struct params_dbl_defn {
    param_types type;		/* DBL */
    double value;
} param_dbl;

typedef union params_defn {
    param_types type;
    param_int intv;		/* INT */
    param_str strv;		/* STR */
    param_pix pixv;		/* PIXEL */
    param_font fontv;		/* FONT */
    param_style stylev;		/* STYLE */
    param_bool boolv;		/* BOOL */
    param_dbl dblv;		/* DBL */
} params;

extern st_table *param_init
_ANSI_ARGS_((Tcl_Interp *interp));

extern void param_destroy
_ANSI_ARGS_((st_table *pt));

extern void param_set
_ANSI_ARGS_((st_table *pt,char *name, param_types type, char *val));

extern void param_reset
 _ANSI_ARGS_((st_table *pt,char *name, char *val));

extern params *param_get
 _ANSI_ARGS_((st_table *pt,char *name, params *val, Tcl_Interp *interp));

extern void param_dump
 _ANSI_ARGS_((st_table *pt));

#ifndef __WIN32__
extern int stricmp _ANSI_ARGS_((char *a, char *b));
#endif

extern params param_temp, *param_temp_ptr;
extern XColor param_null_color; /* x and tk both declare this */
extern char *param_null_tkcolor;
extern char *param_null_tkfont;
extern param_style param_null_style;

/* Some convenience macros */

#define PM_INT(p,name,i)	\
((param_temp_ptr = param_get(p,name, &param_temp,i)) ? \
 param_temp_ptr->intv.value : \
 (abort(), (int) 0))

#define PM_STR(p,name,i)	\
((param_temp_ptr = param_get(p,name, &param_temp,i)) ? \
 param_temp_ptr->strv.value : (abort(), (char *) 0))

#define PM_COLOR(p,name,i)	\
((param_temp_ptr = param_get(p,name, &param_temp, i)) ? \
 param_temp_ptr->pixv.value : (abort(), &param_null_color))

#define PM_TKCOLOR(p,name,i)	\
((param_temp_ptr = param_get(p,name, &param_temp,i))!=(params *)0 ? \
 param_temp_ptr->pixv.tkcolorname : (abort(), param_null_tkcolor))

#define PM_TKFONT(p,name,i)	\
((param_temp_ptr = param_get(p,name, &param_temp,i))!=(params *)0 ? \
 param_temp_ptr->fontv.tkfontname : (abort(), param_null_tkfont ))

#define PM_TKFONTLEN(p,name,i)	\
((param_temp_ptr = param_get(p,name, &param_temp,i))!=(params *)0 ? \
 param_temp_ptr->fontv.tkfontnamelen : (abort(), 0 ))

#define PM_FONT(p,name,i)	\
((param_temp_ptr = param_get(p,name, &param_temp,i)) ? \
 param_temp_ptr->fontv.value : (abort(), (Tk_Font) 0))

#define PM_STYLE(p,name,i)	\
((param_temp_ptr = param_get(p,name, &param_temp,i)) ? \
 param_temp_ptr->stylev : (abort(), param_null_style))

#define PM_BOOL(p,name,i)	\
((param_temp_ptr = param_get(p,name, &param_temp,i)) ? \
 param_temp_ptr->boolv.value : (abort(), 0))

#define PM_DBL(p,name,i)	\
((param_temp_ptr = param_get(p,name, &param_temp,i)) ? \
 param_temp_ptr->dblv.value : (abort(), 0.0))


#endif /* _PARAMS_H_ */
