/*
 * Generic Output Driver for X
 * X version 11
 *
 * This is the primary output driver used by the new X graph
 * to display output to the X server.  It has been factored
 * out of the original xgraph to allow mulitple hardcopy
 * output devices to share xgraph's capabilities.  Note:
 * xgraph is still heavily X oriented.  This is not intended
 * for porting to other window systems.
 */

#include <stdio.h>
#include <assert.h>
#include "tcl.h"
#include "tk.h"
#include "tkxg.h"
#include "params.h"
#include "xgout.h"

#define PADDING 	2
#define SPACE 		10
#define TICKLENGTH	5
#define MAXSEGS		1000

typedef struct attr_set {
    char lineStyle[MAXLS];
    char *lineStyleTkName;
    int lineStyleLen;
    XColor *tkcolor /* pixelValue */ ;
    Pixmap markStyle;
    char *tkcolorname;
} AttrSet;

typedef struct tkxg_state {
   Tcl_Interp *interp;
   st_table *pt;
   AttrSet AllAttrs[MAXATTR];
} tkxg_state;

/*
 * Marker bitmaps
 */

#include "bitmaps/dot.11"

#include "bitmaps/mark1.11"
#include "bitmaps/mark2.11"
#include "bitmaps/mark3.11"
#include "bitmaps/mark4.11"
#include "bitmaps/mark5.11"
#include "bitmaps/mark6.11"
#include "bitmaps/mark7.11"
#include "bitmaps/mark8.11"

/* Sizes exported for marker drawing */
static int dot_w = dot_width;
static int dot_h = dot_height;
static int mark_w = mark1_width;
static int mark_h = mark1_height;

static char *g_string =NULL;
static unsigned int g_size=0,g_count = 0;

void Rebuf()
{
  g_string[0] ='\0';
  g_count--;
}

char *Buf(size)
unsigned size;
{
  assert(g_count==0);
  if (size > g_size) {
    if (g_string!=NULL) {
      free(g_string);
    }
    g_string = (char *)malloc(size+1);
    g_string[size] = '\0';
    g_size = size;
  }
  assert(g_string!=NULL);
  g_count++;
  return g_string;
}


void text_tk(user_state, x, y, text, just, style,class)
void *user_state;               /* Value set in xg_init   */
int x, y;                       /* Text position (pixels) */
char *text;                     /* Null terminated text   */
int just;                       /* Justification (above)  */
int style;                      /* Text style (above)     */
enum xgenum class;
/*
 * This routine should draw text at the indicated position using
 * the indicated justification and style.  The justification refers
 * to the location of the point in reference to the text.  For example,
 * if just is T_LOWERLEFT,  (x,y) should be located at the lower left
 * edge of the text string.
 */
{
    char *jstring, *font, *cmd;
    Tcl_Interp *interp;
    int flen=0, len=0, jlen=0;
    struct tkxg_state *tk_state;
    st_table *pt;

    tk_state = (struct tkxg_state *)user_state;
    interp = tk_state->interp;
    pt = tk_state->pt;
    jstring = NULL;
    if (style == T_TITLE) { 
        font = PM_TKFONT(pt,"TitleFont",interp);
        flen = PM_TKFONTLEN(pt,"TitleFont",interp);
    } else { 
        font = PM_TKFONT(pt,"LabelFont",interp);
        flen = PM_TKFONTLEN(pt,"LabelFont",interp);
    }
    switch (just) {
    case T_CENTER:
	jstring = "center";
        jlen = 6;
	break;
    case T_LEFT:
        jstring = "w";
        jlen = 1;
	break;
    case T_UPPERLEFT:
        jstring = "nw";
        jlen = 2;
	break;
    case T_TOP:
        jstring = "n";
        jlen = 1;
	break;
    case T_UPPERRIGHT:
        jstring = "ne";
        jlen = 2;
	break;
    case T_RIGHT:
        jstring = "e";
        jlen = 1;
	break;
    case T_LOWERRIGHT:
        jstring = "se";
        jlen = 2;
	break;
    case T_BOTTOM:
        jstring = "s";
        jlen = 1;
	break;
    case T_LOWERLEFT:
        jstring = "sw";
        jlen = 2;
	break;
    }
    len = strlen(text);
    len += jlen; /* jstring */
    len += flen; /* font name */
    len += 30; /* room for 3 %d */
    len += 19; /*tkxg_text {}    {}\n*/
    
#if TDEBUG
    fprintf(stdout,"Tkxg_Text {%s} %d %d %s {%s} %d\n",
        text,x,y,jstring,font,class);
#endif
    cmd = Buf(len+1);
    sprintf(cmd,"%s {%s} %d %d %s {%s} %d",
        TKXG_TEXT,text,x,y,jstring,font,class);
    Tcl_GlobalEval(interp,cmd);
    Rebuf();
}


void seg_tk(user_state, ns, segs, width, style, lappr, color, class)
void *user_state;               /* Value set in xg_init */
int ns;                         /* Number of segments   */
XSegment *segs;                 /* X array of segments  */
int width;                      /* Width of lines       */
int style;                      /* See above            */
int lappr;                      /* Line appearence      */
int color;                      /* Line color (if any)  */
enum xgenum class;
/*
 * This routine draws a number of line segments at the points
 * given in `seglist'.  Note that contiguous segments need not share
 * endpoints but often do.  All segments should be `width' devcoords wide
 * and drawn in style `style'.  If `style' is L_VAR,  the parameters
 * `color' and `lappr' should be used to draw the line.  Both
 * parameters vary from 0 to 7.  If the device is capable of
 * color,  `color' varies faster than `style'.  If the device
 * has no color,  `style' will vary faster than `color' and
 * `color' can be safely ignored.  However,  if the
 * the device has more than 8 line appearences,  the two can
 * be combined to specify 64 line style variations.
 * Xgraph promises not to send simultaneously more than the `max_segs' in the
 * xgOut structure passed back from xg_init().
 */
{
    param_style ps;
    int is;
    char *cmd;
    Tcl_Interp *interp;
    struct tkxg_state *tk_state;
    st_table *pt;

    tk_state = (struct tkxg_state *)user_state;
    interp = tk_state->interp;
    pt = tk_state->pt;

    cmd = Buf(2048);

    /* tell the plotter to do line configurations */
    if (style == L_AXIS) {
	ps = PM_STYLE(pt,"GridStyle",interp);
	if (ps.len < 2) {
            sprintf(cmd,"%s %s %d {} %d", TKXG_SEGCON,
                    PM_TKCOLOR(pt,"Foreground",interp),
                    PM_INT(pt,"GridSize",interp),class);
	} else {
            sprintf(cmd,"%s %s %d %s %d", TKXG_SEGCON,
                    PM_TKCOLOR(pt,"Foreground",interp),
                    PM_INT(pt,"GridSize",interp),
                    ps.tkbitname,class);
	}
    } else if (style == L_ZERO) {
	/* Set the color and line style */
	ps = PM_STYLE(pt,"ZeroStyle",interp);
	if (ps.len < 2) {
            sprintf(cmd,"%s %s %d {} %d",TKXG_SEGCON,
                    PM_TKCOLOR(pt,"ZeroColor",interp),
                    PM_INT(pt,"ZeroWidth",interp),class); 
		    /* solid, no stipple */
	} else {
            sprintf(cmd,"%s %s %d %s %d",TKXG_SEGCON,
                    PM_TKCOLOR(pt,"ZeroColor",interp),
                    PM_INT(pt,"ZeroWidth",interp),
                    ps.tkbitname,class);
	}
    } else {
	/* Color and line style vary */
	if (lappr == 0) {
             sprintf(cmd,"%s %s %d {} %d", TKXG_SEGCON,
                     tk_state->AllAttrs[color].tkcolorname,width,class);
                     /* solid */
	} else {
             sprintf(cmd,"%s %s %d %s %d", TKXG_SEGCON,
                     tk_state->AllAttrs[color].tkcolorname, width,
                     tk_state->AllAttrs[lappr].lineStyleTkName,class);
	}
    }
    if (cmd[0] != '\0') {
        Tcl_GlobalEval(interp,cmd);
    }
    for (is = 0 ; is < ns; is++) {
#if TDEBUG
        fprintf(stdout,"Segment %d %d %d %d %d\n", 
                segs[is].x1,
                segs[is].y1,
                segs[is].x2,
                segs[is].y2,class);
#endif
        sprintf(cmd,"%s %d %d %d %d %d", TKXG_SEG,
                segs[is].x1, segs[is].y1,
                segs[is].x2, segs[is].y2,
                class);
        Tcl_GlobalEval(interp,cmd);
    }
    Rebuf();
}




void dot_tk(user_state, x, y, style, type, color, class)
void *user_state;               /* Value set in xg_init    */
int x, y;                       /* Location in pixel units */
int style;                      /* Dot style               */
int type;                       /* Type of marker          */
int color;                      /* Marker color (if any)   */
enum xgenum class;
/*
 * This routine should draw a marker at location `x,y'.  If the
 * style is P_PIXEL,  the dot should be a single pixel.  If
 * the style is P_DOT,  the dot should be a reasonably large
 * dot.  If the style is P_MARK,  it should be a distinguished
 * mark which is specified by `type' (0-7).  If the output
 * device is capable of color,  the marker should be drawn in
 * `color' (0-7) which corresponds with the color for xg_line.
 */
{
    char *cmd;
    Tcl_Interp *interp;
    tkxg_state *tk_state;

    tk_state = (tkxg_state *)user_state;
    interp = tk_state->interp;
    cmd = Buf(2048);

    sprintf(cmd,"%s %d %d %d %d %s %d",TKXG_DOT,
            x,y,style,type,tk_state->AllAttrs[color].tkcolorname,class);
    Tcl_GlobalEval(interp,cmd);
    Rebuf();
}



/* updates the spaces for titles/axes based on font sizes */
void reset_tk(out_info, interp,pt)
xgOut *out_info;                /* Information to set   */
Tcl_Interp *interp;
st_table *pt;
{
    Tk_Font font;
    Tk_FontMetrics fm, *fmptr;
    fmptr = &fm;

    font = PM_FONT(pt,"LabelFont",interp);
    Tk_GetFontMetrics(font,fmptr);
    out_info->axis_width = Tk_TextWidth(font, "8", 1);
    out_info->axis_height = fm.ascent + fm.descent;

    font = PM_FONT(pt,"TitleFont",interp);
    Tk_GetFontMetrics(font,fmptr);
    out_info->title_width = Tk_TextWidth(font, "8", 1);
    out_info->title_height = fm.ascent + fm.descent;
}

void del_tk(user_state)
void *user_state;
{
    free(user_state);
}

void set_tk( out_info, interp,pt)
xgOut *out_info;		/* Information to set   */
Tcl_Interp *interp;
st_table *pt;
/* 
 * Sets some of the common parameters for the tk output device.
 * called only after basic parameter hash.
 * The out_info axis/title is advisory, pending the user
 * specifying other than the defaults.
 */
{
    struct tkxg_state *new_state;
    Tk_Font font;
    Tk_FontMetrics fm, *fmptr;
    fmptr = &fm;

    out_info->dev_flags = ((tkxg_depth > 3) ? D_COLOR : 0);
    out_info->area_w = out_info->area_h = 0; /* Set later */
    out_info->bdr_pad = PADDING;
    out_info->axis_pad = SPACE;
    out_info->legend_pad = 0;
    out_info->tick_len = TICKLENGTH;

    font = PM_FONT(pt,"LabelFont",interp);
    Tk_GetFontMetrics(font,fmptr);
    out_info->axis_width = Tk_TextWidth(font, "8", 1);
    out_info->axis_height = fm.ascent + fm.descent;

    font = PM_FONT(pt,"TitleFont",interp);
    Tk_GetFontMetrics(font,fmptr);
    out_info->title_width = Tk_TextWidth(font, "8", 1);
    out_info->title_height = fm.ascent + fm.descent;

    out_info->max_segs = MAXSEGS;

    out_info->xg_text = text_tk;
    out_info->xg_seg = seg_tk;
    out_info->xg_dot = dot_tk;
    out_info->xg_end = (void (*)()) 0;
    out_info->xg_del = del_tk;
    new_state = (struct tkxg_state *) malloc(sizeof(struct tkxg_state));
    new_state->interp = interp;
    out_info->user_state = (void *) new_state;
}



static void update_styles_and_colors(interp,pt,tk_state)
Tcl_Interp *interp;
st_table *pt;
struct tkxg_state *tk_state;
/*
 * Initializes AllAttrs for the client.
 */
{
    char name[1024];
    unsigned int idx;
    params style_val;

    /* Get attributes out parameters database */
    for (idx = 0;  idx < MAXATTR;  idx++) {
	(void) sprintf(name, "%d.Style", idx);
	(void) param_get(pt,name, &style_val,interp);
	tk_state->AllAttrs[idx].lineStyleLen = style_val.stylev.len;
	tk_state->AllAttrs[idx].lineStyleTkName = style_val.stylev.tkbitname;
	(void) strncpy(tk_state->AllAttrs[idx].lineStyle, 
                       style_val.stylev.dash_list,
		       (unsigned)style_val.stylev.len);
	(void) sprintf(name, "%d.Color", idx);
	tk_state->AllAttrs[idx].tkcolor = PM_COLOR(pt,name,interp);
	tk_state->AllAttrs[idx].tkcolorname = PM_TKCOLOR(pt,name,interp);
    }
}

void init_tk(interp)
Tcl_Interp *interp;
{
    /* Store bitmaps for dots and markers */
    Tk_DefineBitmap(interp,Tk_GetUid("tkxg_dot"),
                    BITDATA(dot_bits),dot_w,dot_h);
    Tk_DefineBitmap(interp,Tk_GetUid("tkxg_mark1"),
                    BITDATA(mark1_bits),mark_w,mark_h);
    Tk_DefineBitmap(interp,Tk_GetUid("tkxg_mark2"),
                    BITDATA(mark2_bits),mark_w,mark_h);
    Tk_DefineBitmap(interp,Tk_GetUid("tkxg_mark3"),
                    BITDATA(mark3_bits),mark_w,mark_h);
    Tk_DefineBitmap(interp,Tk_GetUid("tkxg_mark4"),
                    BITDATA(mark4_bits),mark_w,mark_h);
    Tk_DefineBitmap(interp,Tk_GetUid("tkxg_mark5"),
                    BITDATA(mark5_bits),mark_w,mark_h);
    Tk_DefineBitmap(interp,Tk_GetUid("tkxg_mark6"),
                    BITDATA(mark6_bits),mark_w,mark_h);
    Tk_DefineBitmap(interp,Tk_GetUid("tkxg_mark7"),
                    BITDATA(mark7_bits),mark_w,mark_h);
    Tk_DefineBitmap(interp,Tk_GetUid("tkxg_mark8"),
                    BITDATA(mark8_bits),mark_w,mark_h);
}

void init_X(user_state,pt)
void *user_state;
st_table *pt;
/*
 * Initializes for a canvas drawing sequence.  Sets up drawing attributes
 * by reading values from the parameter database.
 */
{
    struct tkxg_state *tk_state;
    tk_state = (struct tkxg_state *)user_state;
    
    tk_state->pt = pt;
    update_styles_and_colors(tk_state->interp,pt,tk_state);
}

