/*
 * Output Device Information
 *
 * This file contains definitions for output device interfaces
 * to the graphing program xgraph.
 */
#ifndef _XGOUT_H_
#define _XGOUT_H_


/* Passed device option flags */
#define D_DOCU		0x01

/* Returned device capability flags */
#define D_COLOR		0x01

/* Text justifications */
#define T_CENTER	0
#define T_LEFT		1
#define T_UPPERLEFT	2
#define T_TOP		3
#define T_UPPERRIGHT	4
#define T_RIGHT		5
#define T_LOWERRIGHT	6
#define T_BOTTOM	7
#define T_LOWERLEFT	8

/* Text styles */
#define T_AXIS		0
#define T_TITLE		1

/* Line Styles */
#define L_AXIS		0
#define L_ZERO		1
#define L_VAR		2

/* Marker Styles */
#define P_PIXEL		0
#define P_DOT		1
#define P_MARK		2


enum xgenum {
  xg_border = -5,		/* segment border */
  xg_coord = -4,	/* segment grid tick, text coordinate number */
  xg_y =  -3,		/* segment y axis, text y axis */
  xg_x = -2,		/* segment x axis, text x axis */
  xg_title0 = -1,	/* segment zero axis, text title */
  xg_series_0 = 0,	/* series 0 - 63 points, lines, legend text */
  xg_series_1 = 1,
  xg_series_2 = 2,
  xg_series_3 = 3,
  xg_series_4 = 4,
  xg_series_5 = 5,
  xg_series_6 = 6,
  xg_series_7 = 7,
  xg_series_8 = 8,
  xg_series_9 = 9,
  xg_series_10 = 10,
  xg_series_11 = 11,
  xg_series_12 = 12,
  xg_series_13 = 13,
  xg_series_14 = 14,
  xg_series_15 = 15,
  xg_series_16 = 16,
  xg_series_17 = 17,
  xg_series_18 = 18,
  xg_series_19 = 19,
  xg_series_20 = 20,
  xg_series_21 = 21,
  xg_series_22 = 22,
  xg_series_23 = 23,
  xg_series_24 = 24,
  xg_series_25 = 25,
  xg_series_26 = 26,
  xg_series_27 = 27,
  xg_series_28 = 28,
  xg_series_29 = 29,
  xg_series_30 = 30,
  xg_series_31 = 31,
  xg_series_32 = 32,
  xg_series_33 = 33,
  xg_series_34 = 34,
  xg_series_35 = 35,
  xg_series_36 = 36,
  xg_series_37 = 37,
  xg_series_38 = 38,
  xg_series_39 = 39,
  xg_series_40 = 40,
  xg_series_41 = 41,
  xg_series_42 = 42,
  xg_series_43 = 43,
  xg_series_44 = 44,
  xg_series_45 = 45,
  xg_series_46 = 46,
  xg_series_47 = 47,
  xg_series_48 = 48,
  xg_series_49 = 49,
  xg_series_50 = 50,
  xg_series_51 = 51,
  xg_series_52 = 52,
  xg_series_53 = 53,
  xg_series_54 = 54,
  xg_series_55 = 55,
  xg_series_56 = 56,
  xg_series_57 = 57,
  xg_series_58 = 58,
  xg_series_59 = 59,
  xg_series_60 = 60,
  xg_series_61 = 61,
  xg_series_62 = 62,
  xg_series_63 = 63
};

#if __STDC__
#define TEXTARGS (void *user_data, int x, int y, char *text, \
  int just, int style, enum xgenum)
#define LINEARGS (void *user_data, int ns, XSegment *xs, \
  int width, int style, int lappr, int color, enum xgenum)
#define DOTARGS (void *user_data, int x, int y, int style, int type, \
  int color, enum xgenum)
#define DELARGS (void *user_data)
#define ENDARGS ()
#else
#define TEXTARGS ()
#define LINEARGS ()
#define DOTARGS ()
#define ENDARGS ()
#define DELARGS ()
#endif

/* Output device information returned by initialization routine */

typedef struct xg_out {
    int dev_flags;		/* Device characteristic flags           */
    int area_w, area_h;		/* Width and height in pixels            */
    int bdr_pad;		/* Padding from border                   */
    int axis_pad;		/* Extra space around axis labels        */
    int tick_len;		/* Length of tick mark on axis           */
    int legend_pad;		/* Top of legend text to legend line     */
    int axis_width;		/* Width of big character of axis font   */
    int axis_height;		/* Height of big character of axis font  */
    int title_width;		/* Width of big character of title font  */
    int title_height;		/* Height of big character of title font */
    int max_segs;		/* Maximum number of segments in group   */

    void (*xg_text) TEXTARGS ;	/* Draws text at a location              */
    void (*xg_seg) LINEARGS ;	/* Draws a series of segments            */
    void (*xg_dot) DOTARGS ;	/* Draws a dot or marker at a location   */
    void (*xg_end) ENDARGS ;	/* Stops the drawing sequence            */
    void (*xg_del) DELARGS ;	/* Destroys the user_state */

    void *user_state;		/* User supplied data                    */
} xgOut;

#define ERRBUFSIZE	2048

#endif /* xgout.h */
